/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2009 France Telecom R&D
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jndi.interceptors.impl;

import java.lang.reflect.Method;

import org.objectweb.carol.jndi.intercept.ContextInterceptor;
import org.objectweb.jonas.jndi.checker.IResourceCheckerManager;

/**
 * Abstract class for some JOnAS context interceptors.
 * @author Florent Benoit
 */
public abstract class AbsContextInterceptor implements ContextInterceptor {

    /**
     * Resource checker manager.
     */
    private IResourceCheckerManager resourceCheckerManager = null;

    /**
     * Allows to filter JNDI Name in order to enable interceptor.
     * @param jndiName the JNDI name used for the filter
     * @return true if interceptor may be applied
     */
    protected boolean filterOnJndiName(String jndiName) {
        // No filtering: match all JNDI names
        return true;
    }

    /**
     * For a given method, check if the interceptor is applied or not.
     * @param method the method to check
     * @return true if interceptor may be applied, else false
     */
    protected boolean filterOnMethodName(Method method) {
        return "lookup".equals(method.getName()) || "lookupLink".equals(method.getName());
    }

    /**
     * @return the resource checker manager
     */
    public IResourceCheckerManager getResourceCheckerManager() {
        return resourceCheckerManager;
    }

    /**
     * Sets the resource checker manager.
     * @param resourceCheckerManager the given instance
     */
    public void setResourceCheckerManager(IResourceCheckerManager resourceCheckerManager) {
        this.resourceCheckerManager = resourceCheckerManager;
    }

}
